import os

import kaybee_bulma.events
from kaybee_bulma.siteconfig import SiteConfig

__version__ = "0.2.7"
__title__ = "kaybee_bulma"
__description__ = "Bulma-based theme for Kaybee"
__uri__ = "https://github.com/pauleveritt/kaybee_bulma"
__doc__ = __description__ + " <" + __uri__ + ">"

__author__ = "Paul Everitt"
__email__ = "pauleveritt@me.org"

__license__ = "Apache License 2.0"
__copyright__ = "Copyright (c) 2017 Paul Everitt"


def get_path():
    """
    Called from the entry point in setup.py
    """
    # Theme directory is defined as our parent directory
    return os.path.abspath(os.path.dirname(os.path.dirname(__file__)))


def setup(app):
    app.add_config_value('kaybee_bulma_siteconfig', SiteConfig(), 'html')

    return dict(
        version=__version__,
        parallel_read_safe=False
    )




"""

Automatically re-build Sphinx docs while editing, serve docs on an
HTTP port, and also reload any browsers pointed to the docs.

"""

import glob

from livereload import Server, shell
from livereload.watcher import Watcher

sphinx = "env3/bin/python3 env3/bin/sphinx-build -E -b html docs docs/_build"
dist = "/usr/local/bin/npm run dist"
both = dist + '; ' + sphinx


class CustomWatcher(Watcher):
    """ Handle recursive globs with Python 3.5+ globbing  """

    def is_glob_changed(self, path, ignore=None):
        for f in glob.glob(path, recursive=True):
            if self.is_file_changed(f, ignore):
                return True
        return False


def get_server():
    server = Server(watcher=CustomWatcher())
    return server



import inspect
import os
from operator import attrgetter

import kaybee_bulma
from kaybee import SphinxEvent
from kaybee.app import kb
from sphinx.application import Sphinx
from sphinx.jinja2glue import SphinxFileSystemLoader


@kb.event(SphinxEvent.BI, scope='kaybee_bulma')
def handle_builderinit(kb_app: kb, sphinx_app):
    """ Load the resources, types, etc. from the registry

    We can get resources etc. from 3 location: classes in kaybee itself,
    classes in the doc project, and YAML "typedef" files in the doc
    project.
    """

    # Add the root of this theme
    template_bridge = sphinx_app.builder.templates
    f = os.path.join(os.path.dirname(inspect.getfile(kaybee_bulma)),
                     'templates')
    template_bridge.loaders.append(SphinxFileSystemLoader(f))


@kb.event(SphinxEvent.HPC, scope='kaybee_bulma')
def resource_into_html_context(
        kb_app: kb,
        sphinx_app: Sphinx,
        pagename,
        templatename: str,
        context,
        doctree):
    class Site:
        def __init__(self):
            self.config = sphinx_app.config.kaybee_bulma_siteconfig

    # context['site'] = Site()

    context['siteconfig'] = sphinx_app.config.kaybee_bulma_siteconfig

    resources = sphinx_app.env.resources
    resources = [r for r in resources.values() if
                 getattr(r.props, 'in_nav', False) and
                 r.props.in_nav and r.is_published]

    # Sort first by title, then by "weight"
    context['navmenu'] = sorted(resources,
                                key=lambda x: (
                                    x.props.weight, attrgetter('title')(x))
                                )



from pydantic import BaseModel


class Logo(BaseModel):
    img_url: str = None
    img_file: str = None
    alt: str = None


class SocialMedia(BaseModel):
    twitter: str = None
    github: str = None


class SiteConfig(BaseModel):
    logo: Logo = None
    social_media: SocialMedia = None
    copyright = 'All Rights Reserved'
    feed_url: str = None
    favicon: str = None
    is_debug = False
